@@ -1,6 +1,3 @@
-# Shiny objects if the widget sets elementId, so we
-# need to detect it.  Thanks to Joe Cheng for suggesting this code.
-inShiny <- function() !is.null(getDefaultReactiveDomain())
 
 rmarkdownOutput <- function() {
   if (requireNamespace("rmarkdown", quietly = TRUE)) {
@@ -199,12 +196,17 @@ processUpstream <- function(upstream, elementId = NULL, playerId = NULL) {
 }
 
 asRow <- function(..., last = NA, height = NULL, colsize = 1) {
+	if (!requireNamespace("manipulateWidget", quietly = TRUE)) {
+		warning("asRow requires the 'manipulateWidget' package.", call. = FALSE)
+		last <- NA
+	}
   args <- list(...)
-  if (length(args) == 1 
-      && inherits(args[[1]], "combineWidgets")) {
+  if ((length(args) == 1 
+      && inherits(args[[1]], "combineWidgets"))
+  		|| !requireNamespace("manipulateWidget", quietly = TRUE)) {
     orig <- args[[1]]
   } else {
-    orig <- do.call(combineWidgets, c(args, list(ncol = 1, rowsize = getHeights(args))))
+    orig <- do.call(manipulateWidget::combineWidgets, c(args, list(ncol = 1, rowsize = getHeights(args))))
   }
   origlen <- length(orig$widgets)
   for (i in seq_len(origlen))
@@ -226,9 +228,11 @@ asRow <- function(..., last = NA, height = NULL, colsize = 1) {
   
   orig$params$rowsize <- c(origRowsizes[keep], height)
 
-  row <- do.call(combineWidgets, c(orig$widgets[inrow], list(nrow = 1, 
+  if (requireNamespace("manipulateWidget", quietly = TRUE)) {
+    row <- do.call(manipulateWidget::combineWidgets, c(orig$widgets[inrow], list(nrow = 1, 
                                                              colsize = colsize)))
-  orig$widgets <- c(orig$widgets[keep], list(row))
+    orig$widgets <- c(orig$widgets[keep], list(row))
+  }
   orig
 }
 
@@ -237,12 +241,16 @@ newElementId <- function(prefix)
 
 knitrNeedsSnapshot <- function(options = knitr::opts_current$get()) {
   if (!is.null(options$snapshot))
-    options$snapshot
-  else {
-    pandocTo <- opts_knit$get("rmarkdown.pandoc.to")
-    if (!length(pandocTo)) pandocTo <- ""
-    pandocTo %in% c("latex", "gsm")
-  }
+    return(options$snapshot)
+  if (isFALSE(options$screenshot.force))
+    return(FALSE)
+  force <- isTRUE(options$screenshot.force)
+  fmt <- pandoc_to()
+  if (!length(fmt) || force)
+    return(TRUE)
+  html_format <- fmt %in% c("html", "html4", "html5", "revealjs", 
+                            "s5", "slideous", "slidy")
+  !html_format
 }
 
 rglwidget <- local({
@@ -255,7 +263,8 @@ rglwidget <- local({
            minimal = TRUE, 
            webgl,
            snapshot,
-           shinyBrush = NULL, ...) {
+           shinyBrush = NULL, ...,
+           oldConvertBBox = FALSE) {
     
   if (missing(snapshot)) {
     if (missing(webgl)) {
@@ -327,7 +336,8 @@ rglwidget <- local({
     height <- CSStoPixels(height)
   x <- convertScene(x, width, height,
                    elementId = elementId, 
-                   webgl = webgl, snapshot = snapshot)
+                   webgl = webgl, snapshot = snapshot,
+                   oldConvertBBox = oldConvertBBox)
   
   upstream <- processUpstream(controllers, elementId = elementId)
   
@@ -357,103 +367,17 @@ rglwidget <- local({
   }
     
   if (is.list(upstream$objects)) {
-    do.call(combineWidgets, c(upstream$objects, 
+  	if (requireNamespace("manipulateWidget", quietly = TRUE))
+      result <- do.call(manipulateWidget::combineWidgets, c(upstream$objects, 
                               list(result, 
                                    rowsize = c(upstream$rowsizes, height), 
                                    ncol = 1)))
-  } else
-    result
+  	else 
+  		warning("Combining widgets requires the 'manipulateWidget' package.", call. = FALSE)
+  }
+  result
 }})
 
-#' Widget output function for use in Shiny
-#'
-#' @export
-rglwidgetOutput <- function(outputId, width = '512px', height = '512px') {
-  shinyWidgetOutput(outputId, 'rglWebGL', width, height, package = 'rgl')
-}
-
-#' Widget render function for use in Shiny
-#'
-#' @export
-renderRglwidget <- function(expr, env = parent.frame(), quoted = FALSE, outputArgs = list()) {
-  if (!quoted) expr <- substitute(expr)  # force quoted
-  markRenderFunction(rglwidgetOutput,
-                     shinyRenderWidget(expr, rglwidgetOutput, env, quoted = TRUE),
-  		     outputArgs = outputArgs)
-}
-
-shinySetPar3d <- function(..., session,
-                          subscene = currentSubscene3d(cur3d())) {
-  if (!requireNamespace("shiny", quietly = TRUE))
-    stop("function requires shiny")
-  args <- list(...)
-  argnames <- names(args)
-  if (length(args) == 1 && is.null(argnames)) {
-    args <- args[[1]]
-  }
-  # We might have been passed modified shinyGetPar3d output;
-  # clean it up.
-  args[["subscene"]] <- NULL
-  args[["tag"]] <- NULL
-  argnames <- names(args)
-  
-  if (is.null(argnames) || any(argnames == ""))
-    stop("Parameters must all be named")
-  
-  badargs <- argnames[!(argnames %in% .Par3d) | argnames %in% .Par3d.readonly]
-  if (length(badargs))
-    stop("Invalid parameter(s): ", badargs)
-  
-  for (arg in argnames) {
-    session$sendCustomMessage("shinySetPar3d", 
-                              list(subscene = subscene,
-                                   parameter = arg,
-                                   value = args[[arg]]))
-  }
-}
-
-shinyGetPar3d <- function(parameters, session,
-                          subscene = currentSubscene3d(cur3d()),
-                          tag = "") {
-  badargs <- parameters[!(parameters %in% .Par3d)]
-  if (length(badargs))
-    stop("invalid parameter(s): ", badargs)
-  session$sendCustomMessage("shinyGetPar3d",
-                              list(tag = tag, subscene = subscene, 
-                                   parameters = parameters))
-}
-
-convertShinyPar3d <- function(par3d, ...) {
-  for (parm in c("modelMatrix", "projMatrix", "userMatrix", "userProjection"))
-    if (!is.null(par3d[[parm]]))
-      par3d[[parm]] <- matrix(unlist(par3d[[parm]]), 4, 4)
-  for (parm in c("mouseMode", "observer", "scale", "viewport", "bbox", "windowRect"))
-    if (!is.null(par3d[[parm]]))
-      par3d[[parm]] <- unlist(par3d[[parm]])
-  par3d
-}
-
-shinyResetBrush <- function(session, brush) {
-  session$sendCustomMessage("resetBrush", brush)
-}
-
-convertShinyMouse3d <- function(mouse3d, ...) {
-  for (parm in c("model", "proj")) 
-    if (!is.null(mouse3d[[parm]]))
-      mouse3d[[parm]] <- matrix(unlist(mouse3d[[parm]]), 4, 4)
-    
-  if (length(unlist(mouse3d$view)) == 4) 
-    mouse3d$view <- structure(unlist(mouse3d$view),
-                              names = c("x", "y", "width", "height"))
-  if (all(c("p1", "p2") %in% names(mouse3d$region)))
-    mouse3d$region <- with(mouse3d$region,
-                           c(x1 = (p1$x + 1)/2,
-                             y1 = (p1$y + 1)/2,
-                             x2 = (p2$x + 1)/2,
-                             y2 = (p2$y + 1)/2))
-  structure(mouse3d, class = "rglMouseSelection")
-}
-
 print.rglMouseSelection <- function(x, verbose = FALSE, ...) {
   if (!is.null(x$region)) {
     cat("Mouse selection:\n")
@@ -510,7 +434,7 @@ CanvasMatrixDependency <- makeDependency("CanvasMatrix4",
                                          src = "htmlwidgets/lib/CanvasMatrix",
                                          script = "CanvasMatrix.src.js",
                                          package = "rgl",
-                                         debugging = nchar(Sys.getenv("RGL_DEBUGGING", "")) > 0)
+                                         debugging = isTRUE(as.logical(Sys.getenv("RGL_DEBUGGING", "FALSE"))))
 
 rglDependency <- makeDependency("rglwidgetClass", 
                       src = "htmlwidgets/lib/rglClass",
@@ -526,7 +450,9 @@ rglDependency <- makeDependency("rglwidgetClass",
                                  "draw.src.js",
                                  "controls.src.js",
                                  "selection.src.js",
-                                 "rglTimer.src.js"),
+                                 "rglTimer.src.js",
+                                 "pretty.src.js",
+                                 "axes.src.js"),
                       stylesheet = "rgl.css",
                       package = "rgl",
-                      debugging = nchar(Sys.getenv("RGL_DEBUGGING", "")) > 0)
+                      debugging = isTRUE(as.logical(Sys.getenv("RGL_DEBUGGING", "FALSE"))))
