@@ -333,32 +333,34 @@ static Side side[6] = {
 };
 
 struct Edge{
-  Edge(int in_from, int in_to, Vertex4 in_dir) : from(in_from), to(in_to), dir(in_dir) { }
+  Edge(int in_from, int in_to, Vertex4 in_dir, Vertex3 in_code) : from(in_from), to(in_to), dir(in_dir), code(in_code) { }
   int from, to;
   Vertex4 dir;
+  Vertex3 code;
 };
 
 static Edge xaxisedge[4] = { 
-  Edge( 5,4, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f) ), 
-  Edge( 0,1, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f) ),
-  Edge( 6,7, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f) ),
-  Edge( 3,2, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f) )
+  Edge( 5,4, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f), Vertex3( 0.0f, -1.0f,  1.0f) ), 
+  Edge( 0,1, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f), Vertex3( 0.0f, -1.0f, -1.0f) ),
+  Edge( 6,7, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f), Vertex3( 0.0f,  1.0f,  1.0f) ),
+  Edge( 3,2, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f), Vertex3( 0.0f,  1.0f, -1.0f) )
 };
+
 static Edge yaxisedge[8] = { 
-  Edge( 5,7, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f) ),
-  Edge( 7,5, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f) ), 
-  Edge( 6,4, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f) ), 
-  Edge( 4,6, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f) ), 
-  Edge( 2,0, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f) ), 
-  Edge( 0,2, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f) ),
-  Edge( 3,1, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f) ), 
-  Edge( 1,3, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f) )
+  Edge( 5,7, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f), Vertex3( 1.0f, 0.0f, 1.0f) ),
+  Edge( 7,5, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f), Vertex3( 1.0f, 0.0f, 1.0f) ), 
+  Edge( 6,4, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f), Vertex3(-1.0f, 0.0f, 1.0f) ), 
+  Edge( 4,6, Vertex4( 0.0f, 0.0f, 1.0f, 0.0f), Vertex3(-1.0f, 0.0f, 1.0f) ), 
+  Edge( 2,0, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f), Vertex3(-1.0f, 0.0f, -1.0f)  ), 
+  Edge( 0,2, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f), Vertex3(-1.0f, 0.0f, -1.0f)  ),
+  Edge( 3,1, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f), Vertex3( 1.0f, 0.0f, -1.0f)  ), 
+  Edge( 1,3, Vertex4( 0.0f, 0.0f,-1.0f, 0.0f), Vertex3( 1.0f, 0.0f, -1.0f)  )
 };
 static Edge zaxisedge[4] = { 
-  Edge( 1,5, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f) ), 
-  Edge( 4,0, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f) ), 
-  Edge( 7,3, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f) ), 
-  Edge( 2,6, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f) ) 
+  Edge( 1,5, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f), Vertex3( 1.0f,-1.0f, 0.0f)  ), 
+  Edge( 4,0, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f), Vertex3(-1.0f,-1.0f, 0.0f)  ), 
+  Edge( 7,3, Vertex4( 1.0f, 0.0f, 0.0f, 0.0f), Vertex3( 1.0f, 1.0f, 0.0f)  ), 
+  Edge( 2,6, Vertex4(-1.0f, 0.0f, 0.0f, 0.0f), Vertex3(-1.0f, 1.0f, 0.0f)  ) 
 };
 
 
@@ -368,10 +370,10 @@ Material BBoxDeco::defaultMaterial( Color(0.6f,0.6f,0.6f,0.5f), Color(1.0f,1.0f,
 BBoxDeco::BBoxDeco(Material& in_material, AxisInfo& in_xaxis, AxisInfo& in_yaxis, AxisInfo& in_zaxis, float in_marklen_value, bool in_marklen_fract,
                    float in_expand, bool in_front)
 : SceneNode(BBOXDECO), material(in_material), xaxis(in_xaxis), yaxis(in_yaxis), zaxis(in_zaxis), marklen_value(in_marklen_value), marklen_fract(in_marklen_fract),
-#ifndef RGL_NO_OPENGL
-  expand(in_expand), 
+  expand(in_expand), draw_front(in_front)
+#ifndef RGL_NO_OPENGL  
+  , axisBusy(false)
 #endif
-  draw_front(in_front)
 {
   material.colors.recycle(2);
 }
@@ -395,25 +397,25 @@ AABox BBoxDeco::getBoundingBox(const AABox& in_bbox) const
   return bbox2;
 }
 
-void BBoxDeco::render(RenderContext* renderContext)
-{
-#ifndef RGL_NO_OPENGL  
-  AABox bbox = renderContext->subscene->getBoundingBox();
-
-  if (bbox.isValid()) {
+struct BBoxDeco::BBoxDecoImpl {
   
+  static Edge* chooseEdge(RenderContext* renderContext, BBoxDeco& bboxdeco, int coord) 
+  {
+    
+    AABox bbox = renderContext->subscene->getBoundingBox();
+    
     Vertex center = bbox.getCenter();
-    bbox += center + (bbox.vmin - center)*expand;
-    bbox += center + (bbox.vmax - center)*expand;
-
-    // Sphere bsphere(bbox);
-
-    glPushAttrib(GL_ENABLE_BIT);
-
+    bbox += center + (bbox.vmin - center)*bboxdeco.expand;
+    bbox += center + (bbox.vmax - center)*bboxdeco.expand;
+    
+    // edge adjacent matrix
+    
+    int adjacent[8][8] = { { 0 } };
+    
     int i,j;
-
+    
     // vertex array:
-
+    
     Vertex4 boxv[8] = {
       Vertex4( bbox.vmin.x, bbox.vmin.y, bbox.vmin.z ),
       Vertex4( bbox.vmax.x, bbox.vmin.y, bbox.vmin.z ),
@@ -424,254 +426,412 @@ void BBoxDeco::render(RenderContext* renderContext)
       Vertex4( bbox.vmin.x, bbox.vmax.y, bbox.vmax.z ),
       Vertex4( bbox.vmax.x, bbox.vmax.y, bbox.vmax.z )
     };
-
+    
     Vertex4 eyev[8];
-
+    
     // transform vertices: used for edge distance criterion and text justification
-
+    
     Matrix4x4 modelview(renderContext->subscene->modelMatrix);
-
+    
     for(i=0;i<8;i++)
       eyev[i] = modelview * boxv[i];
- 
+    
+    for(i=0;i<6;i++) {
+      
+      const Vertex4 q = modelview * side[i].normal;
+      const Vertex4 view(0.0f,0.0f,1.0f,0.0f);
+      
+      float cos_a = view * q;
+      
+      const bool front = (cos_a >= 0.0f) ? true : false;
+      
+      if (bboxdeco.draw_front || !front) {
+        
+        for(j=0;j<4;j++) {
+          
+          if (!front) {
+            // modify adjacent matrix
+            
+            int from = side[i].vidx[j];
+            int to   = side[i].vidx[(j+1)%4];
+            
+            adjacent[from][to] = 1;
+          }          
+          
+        }
+        
+      }
+    }
+    
+    AxisInfo*  axis;
+    Edge*  axisedge;
+    int    nedges;
+    
+    switch(coord)
+    {
+      case 0:
+        axis     = &(bboxdeco.xaxis);       
+        axisedge = xaxisedge;
+        nedges   = 4;
+        break;
+      case 1:
+        axis     = &(bboxdeco.yaxis);
+        axisedge = yaxisedge;
+        nedges   = 8;
+        break;
+      case 2:
+      default:
+        axis     = &(bboxdeco.zaxis);
+        axisedge = zaxisedge;
+        nedges   = 4;
+        break;
+    }
+    
+    // search z-nearest contours
+    
+    float d = FLT_MAX;
+    Edge* edge = NULL;
+    
+    for(j=0;j<nedges;j++) {
+      
+      int from = axisedge[j].from;
+      int to   = axisedge[j].to;
+      
+      if ((adjacent[from][to] == 1) && (adjacent[to][from] == 0)) {
+        
+        // found contour
+        
+        float dtmp = -(eyev[from].z + eyev[to].z)/2.0f;
+        
+        if (dtmp < d) {
+          
+          // found near contour
+          
+          d = dtmp;
+          edge = &axisedge[j];
+          
+        }
+        
+      } 
+      
+    }
+    return edge;
+  }
+  
+  static Edge* fixedEdge(Material* material)
+  {
+    Edge* axisedge;
+    int i,j, lim, coord = material->marginCoord;
+    bool match;
+    switch(coord) {
+      case 0: 
+        axisedge = xaxisedge;
+        lim = 4;
+        break;
+      case 1: 
+        axisedge = yaxisedge; 
+        lim = 8;
+        break;
+      case 2: 
+        axisedge = zaxisedge;
+        lim = 4;
+        break;
+    }
+    for (j = 0; j < lim; j++) {
+      match = true;
+      for (i = 0; i < 3; i++)
+        if (coord != i && axisedge[j].code[i] != material->edge[i]) {
+          match = false;
+          break;
+        }
+      if (match)
+        return &axisedge[j];
+    }
+    error("fixedEdge: material->floating=%d marginCoord=%d edge=%d %d %d\n", material->floating,
+             material->marginCoord, material->edge[0], material->edge[1], material->edge[2]);
+    return axisedge;
+  }
+  
+  static void setMarginParameters(RenderContext* renderContext, BBoxDeco& bboxdeco, Material* material,
+                                  int *at, int* line, int* level,
+                                  Vec3* trans, Vec3* scale) {
+    Edge* edge;
+    int j;
+    *at = material->marginCoord;
+    if (material->floating)
+      edge = BBoxDecoImpl::chooseEdge(renderContext, bboxdeco, *at);
+    else
+      edge = BBoxDecoImpl::fixedEdge(material);
+    if (!edge) {
+      *at = NA_INTEGER;
+      return;
+    }
+    for (j = 0; j < 3; j++) {
+      if (edge->dir[j] != 0) {
+        *line = j;
+        break;
+      }
+    }
+    *level = 2;
+    for (j = 0; j < 2; j++) {
+      if (*at != j && *line != j) {
+        *level = j;
+        break;
+      }
+    }
+    /* Set up translation and scaling */
+    AABox bbox = renderContext->subscene->getBoundingBox();
+    Vertex marklen = bboxdeco.getMarkLength(bbox);
+    for (j = 0; j < 3; j++) {
+      if (j != *at) {
+        int e = 1;
+        if (material->floating && edge->code[j] < 0)
+          e = -1;
+        e = e*material->edge[j];
+        (*trans)[j] = e == 1 ? bbox.vmax[j] : bbox.vmin[j];
+        (*scale)[j] = marklen[j]*e;
+      } else {
+        (*trans)[j] = 0.0;
+        (*scale)[j] = 1.0;
+      }
+    }
+  };
+};
+
+void BBoxDeco::render(RenderContext* renderContext)
+{
+#ifndef RGL_NO_OPENGL  
+  AABox bbox = renderContext->subscene->getBoundingBox();
+  
+  if (bbox.isValid()) {
+    
+    glPushAttrib(GL_ENABLE_BIT);
+    
+    int i,j;
+    
+    // vertex array:
+    
+    Vertex4 boxv[8] = {
+      Vertex4( bbox.vmin.x, bbox.vmin.y, bbox.vmin.z ),
+      Vertex4( bbox.vmax.x, bbox.vmin.y, bbox.vmin.z ),
+      Vertex4( bbox.vmin.x, bbox.vmax.y, bbox.vmin.z ),
+      Vertex4( bbox.vmax.x, bbox.vmax.y, bbox.vmin.z ),
+      Vertex4( bbox.vmin.x, bbox.vmin.y, bbox.vmax.z ),
+      Vertex4( bbox.vmax.x, bbox.vmin.y, bbox.vmax.z ),
+      Vertex4( bbox.vmin.x, bbox.vmax.y, bbox.vmax.z ),
+      Vertex4( bbox.vmax.x, bbox.vmax.y, bbox.vmax.z )
+    };
+    
+    // 
+    // // transform vertices: used for edge distance criterion and text justification
+    // 
+    Matrix4x4 modelview(renderContext->subscene->modelMatrix);
+    
     // setup material
     
     material.beginUse(renderContext);
-
+    
     if (material.line_antialias || material.isTransparent()) {
       // SETUP BLENDING
       if (renderContext->gl2psActive == GL2PS_NONE) 
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
       else
-	gl2psBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
+        gl2psBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
       // ENABLE BLENDING
       glEnable(GL_BLEND);
       
     }
     
-    // edge adjacent matrix
-
-    int adjacent[8][8] = { { 0 } };
-
     // draw back faces
-    // construct adjacent matrix
-
+    
     glBegin(GL_QUADS);
-
+    
     for(i=0;i<6;i++) {
-
+      
       const Vertex4 q = modelview * side[i].normal;
       const Vertex4 view(0.0f,0.0f,1.0f,0.0f);
       
       float cos_a = view * q;
-
+      
       const bool front = (cos_a >= 0.0f) ? true : false;
-
+      
       if (draw_front || !front) {
-
+        
         // draw face
-
+        
         glNormal3f(side[i].normal.x, side[i].normal.y, side[i].normal.z);
-
+        
         for(j=0;j<4;j++) {
-
-	  if (!front) {
-            // modify adjacent matrix
-
-            int from = side[i].vidx[j];
-            int to   = side[i].vidx[(j+1)%4];
-
-            adjacent[from][to] = 1;
-          }          
           
           // feed vertex
-
+          
           Vertex4& v = boxv[ side[i].vidx[j] ];
           glVertex3f(v.x, v.y, v.z);
-
+          
         }
-
+        
       }
     }
-
+    
     glEnd();
-
+    
     // setup mark length
-
+    
     Vertex marklen = getMarkLength(bbox);
-
-
+    
+    
     // draw axis and tickmarks
     // find contours
     
     glDisable(GL_LIGHTING);
-
+    
     material.useColor(1);
-
+    
     for(i=0;i<3;i++) {
-
       Vertex4 v;
       AxisInfo*  axis;
-      Edge*  axisedge;
-      int    nedges;
       float* valueptr;
       float  low, high;
-
       switch(i)
       {
         case 0:
           axis     = &xaxis;       
-          axisedge = xaxisedge;
-          nedges   = 4;
           valueptr = &v.x;
           low      = bbox.vmin.x;
           high     = bbox.vmax.x;
           break;
         case 1:
           axis     = &yaxis;
-          axisedge = yaxisedge;
-          nedges   = 8;
           valueptr = &v.y;
           low      = bbox.vmin.y;
           high     = bbox.vmax.y;
           break;
         case 2:
-	default:
+        default:
           axis     = &zaxis;
-          axisedge = zaxisedge;
-          nedges   = 4;
           valueptr = &v.z;
           low      = bbox.vmin.z;
           high     = bbox.vmax.z;
           break;
       }
-
       if (axis->mode == AXIS_NONE)
         continue;
-
-      // search z-nearest contours
       
-      float d = FLT_MAX;
-      Edge* edge = NULL;
-
-      for(j=0;j<nedges;j++) {
-  
-        int from = axisedge[j].from;
-        int to   = axisedge[j].to;
-
-        if ((adjacent[from][to] == 1) && (adjacent[to][from] == 0)) {
-
-          // found contour
-          
-          float dtmp = -(eyev[from].z + eyev[to].z)/2.0f;
-
-          if (dtmp < d) {
-  
-            // found near contour
-
-            d = dtmp;
-            edge = &axisedge[j];
-
+      Edge* edge = BBoxDecoImpl::chooseEdge(renderContext, *this, i); 
+      
+      if (axis->mode == AXIS_USER) {
+        
+        if (!axisBusy) {
+          axisBusy = true;
+          if (axisCallback[i]) {
+            int e[3];
+            if (edge) {
+              e[0] = edge->code[0];
+              e[1] = edge->code[1];
+              e[2] = edge->code[2];
+            } else{
+              e[0] = 0;
+              e[1] = 0;
+              e[2] = 0;
+            }
+            axisCallback[i](axisData[i], i, e);
+            axisBusy = false;
           }
-
-        } 
-
-      }
-
-      if (edge) {
-
-        v = boxv[edge->from];
-
-        switch (axis->mode) {
-          case AXIS_CUSTOM:
-            {
-              // draw axis and tickmarks
-
-              StringArrayIterator iter(&axis->textArray);
-
-
-              for (iter.first(), j=0; (j<axis->nticks) && (!iter.isDone());j++, iter.next()) {
-
-                float value = axis->ticks[j];
-
-                // clip marks
-
-                if ((value >= low) && (value <= high)) {
-                
-                  String string = iter.getCurrent();
-                  *valueptr = value;
-                  axis->draw(renderContext, v, edge->dir, modelview, marklen, string);
-                }
-
-              }
-            }
-            break;
-          case AXIS_LENGTH:
-            {
-              float delta = (axis->len>1) ? (high-low)/((axis->len)-1) : 0;
-
-              for(int k=0;k<axis->len;k++)
-              {
-                float value = low + delta * (float)k;
-                
-                *valueptr = value;
-
-                char text[32];
-                sprintf(text, "%.4g", value);
-
-                String string(static_cast<int>(strlen(text)),text);
-
-                axis->draw(renderContext, v, edge->dir, modelview, marklen, string);
-              }
-            }
-            break;
-          case AXIS_UNIT:
-            {
-              float value =  ( (float) ( (int) ( ( low+(axis->unit-1) ) / (axis->unit) ) ) ) * (axis->unit);
-              while(value < high) {
-
-                *valueptr = value;
-
-                char text[32];
-                sprintf(text, "%.4g", value);
-
-                String s (static_cast<int>(strlen(text)),text);
-
-                axis->draw(renderContext, v, edge->dir, modelview, marklen, s );
-
-                value += axis->unit;
-              }
-            }
-            break;
-          case AXIS_PRETTY:
-            {
-              /* These are the defaults from the R pretty() function, except min_n is 3 */
-              double lo=low, up=high, shrink_sml=0.75, high_u_fact[2];
-              int ndiv=axis->len, min_n=3, eps_correction=0;
-              
-              high_u_fact[0] = 1.5;
-              high_u_fact[1] = 2.75;
-              axis->unit = static_cast<float>(R_pretty0(&lo, &up, &ndiv, min_n, shrink_sml, high_u_fact, 
-                                     eps_correction, 0));
-              
-              for (int i=(int)lo; i<=up; i++) {
-                float value = i*axis->unit;
-		if (value >= low && value <= high) {
-                  *valueptr = value;
-
-                  char text[32];
-                  sprintf(text, "%.4g", value);
-
-                  String s (static_cast<int>(strlen(text)),text);
-
-                  axis->draw(renderContext, v, edge->dir, modelview, marklen, s );
-		}
-              }
-            }
-            break;
-            
         }
-      }
+      } else if (edge) {
+        
+        v = boxv[edge->from];
+        
+        switch (axis->mode) {
+        case AXIS_CUSTOM:
+        {
+          // draw axis and tickmarks
+          
+          StringArrayIterator iter(&axis->textArray);
+          
+          
+          for (iter.first(), j=0; (j<axis->nticks) && (!iter.isDone());j++, iter.next()) {
+            
+            float value = axis->ticks[j];
+            
+            // clip marks
+            
+            if ((value >= low) && (value <= high)) {
+              
+              String string = iter.getCurrent();
+              *valueptr = value;
+              axis->draw(renderContext, v, edge->dir, modelview, marklen, string);
+            }
+            
+          }
+        }
+        break;
+        case AXIS_LENGTH:
+        {
+          float delta = (axis->len>1) ? (high-low)/((axis->len)-1) : 0;
+  
+          for(int k=0;k<axis->len;k++)
+          {
+            float value = low + delta * (float)k;
+  
+            *valueptr = value;
+  
+            char text[32];
+            sprintf(text, "%.4g", value);
+              
+            String string(static_cast<int>(strlen(text)),text);
+              
+            axis->draw(renderContext, v, edge->dir, modelview, marklen, string);
+          }
+        }
+        break;
+        case AXIS_UNIT:
+        {
+          float value =  ( (float) ( (int) ( ( low+(axis->unit-1) ) / (axis->unit) ) ) ) * (axis->unit);
+          while(value < high) {
+                
+            *valueptr = value;
+                
+            char text[32];
+            sprintf(text, "%.4g", value);
+                
+            String s (static_cast<int>(strlen(text)),text);
+                
+            axis->draw(renderContext, v, edge->dir, modelview, marklen, s );
+                
+            value += axis->unit;
+          }
+        }
+        break;
+        case AXIS_PRETTY:
+        {
+        /* These are the defaults from the R pretty() function, except min_n is 3 */
+          double lo=low, up=high, shrink_sml=0.75, high_u_fact[2];
+          int ndiv=axis->len, min_n=3, eps_correction=0;
+                
+          high_u_fact[0] = 1.5;
+          high_u_fact[1] = 2.75;
+          axis->unit = static_cast<float>(R_pretty0(&lo, &up, &ndiv, min_n, shrink_sml, high_u_fact, 
+                                                    eps_correction, 0));
+                
+          for (int i=(int)lo; i<=up; i++) {
+            float value = i*axis->unit;
+            if (value >= low && value <= high) {
+              *valueptr = value;
+                    
+              char text[32];
+              sprintf(text, "%.4g", value);
+                    
+              String s (static_cast<int>(strlen(text)),text);
+                    
+              axis->draw(renderContext, v, edge->dir, modelview, marklen, s );
+            }
+          }
+        }
+        break;
+        }
+        
+       }
     }
     material.endUse(renderContext);
     glPopAttrib();
@@ -679,6 +839,68 @@ void BBoxDeco::render(RenderContext* renderContext)
 #endif  
 }
 
+Vec3 BBoxDeco::marginVecToDataVec(Vec3 marginvec, RenderContext* renderContext, Material* material) {
+  /* Create permutation to map at, line, pos to x, y, z */
+  int at = 0, line = 0, level = 0; /* initialize to suppress warning */
+  Vec3 trans, scale;
+  BBoxDecoImpl::setMarginParameters(renderContext, *this, material,
+        &at, &line, &level,
+        &trans, &scale); 
+  if (at == NA_INTEGER) return Vertex(NA_REAL, NA_REAL, NA_REAL);
+  /* It might make more sense to do this by
+   * modifying the MODELVIEW matrix, but 
+   * I couldn't get that right for some reason...
+   */
+  Vertex result;
+  AABox bbox = renderContext->subscene->getBoundingBox();
+  if (marginvec.missing())
+    result[at] = (bbox.vmin[at] + bbox.vmax[at])/2.0;
+  else if (marginvec.x == -INFINITY)
+    result[at] = bbox.vmin[at];
+  else if (marginvec.x == INFINITY)
+    result[at] = bbox.vmax[at];
+  else
+    result[at] = marginvec.x*scale[at] + trans[at];
+  result[line] = marginvec.y*scale[line] + trans[line];
+  result[level] = marginvec.z*scale[level] + trans[level];
+  return result;
+}
+
+Vec3 BBoxDeco::marginNormalToDataNormal(Vec3 marginvec, RenderContext* renderContext, Material* material) {
+  int at=0, line=0, level=0; /* initialize to suppress warning */
+  Vec3 trans, scale;
+  BBoxDecoImpl::setMarginParameters(renderContext, *this, material,
+                                    &at, &line, &level,
+                                    &trans, &scale); 
+  if (at == NA_INTEGER)
+    return Vertex(NA_REAL, NA_REAL, NA_REAL);
+  Vertex result;
+  result[at] = marginvec.x/scale[at];
+  result[line] = marginvec.y/scale[line];
+  result[level] = marginvec.z/scale[level];
+  return result;
+}
+
+void BBoxDeco::setAxisCallback(userAxisPtr fn, void* user, int axis)
+{
+  axisCallback[axis] = fn;
+  axisData[axis] = user;
+  switch(axis) {
+    case 0: xaxis.mode = AXIS_USER;
+            break;
+    case 1: yaxis.mode = AXIS_USER;
+            break;
+    case 2: zaxis.mode = AXIS_USER;
+            break;
+  }
+}
+
+void BBoxDeco::getAxisCallback(userAxisPtr *fn, void** user, int axis)
+{
+  *fn = axisCallback[axis];
+  *user = axisData[axis];
+}
+
 int BBoxDeco::getAttributeCount(AABox& bbox, AttribID attrib) 
 {
   switch (attrib) {    
@@ -696,7 +918,9 @@ int BBoxDeco::getAttributeCount(AABox& bbox, AttribID attrib)
     case COLORS:
       return material.colors.getLength();
     case FLAGS:
-      return 1;
+      return 2;
+    case AXES:
+      return 5;
   }
   return SceneNode::getAttributeCount(bbox, attrib);
 }
@@ -761,7 +985,26 @@ void BBoxDeco::getAttribute(AABox& bbox, AttribID attrib, int first, int count,
       }
       return;
     case FLAGS:
-      *result++ = (double) draw_front;	
+      *result++ = (double) draw_front;
+      *result++ = (double) marklen_fract;
+      break;  // there could be more flags, so fall through...
+    case AXES:
+      *result++ = xaxis.mode;
+      *result++ = yaxis.mode;
+      *result++ = zaxis.mode;
+      *result++ = xaxis.unit;
+      *result++ = yaxis.unit;
+      *result++ = zaxis.unit;
+      *result++ = xaxis.len;
+      *result++ = yaxis.len;
+      *result++ = zaxis.len;
+      *result++ = marklen_value;
+      *result++ = marklen_value;
+      *result++ = marklen_value;
+      *result++ = expand;
+      *result++ = expand;
+      *result++ = expand;
+      return;
     }
     SceneNode::getAttribute(bbox, attrib, first, count, result);
   }
@@ -798,6 +1041,7 @@ String BBoxDeco::getTextAttribute(AABox& bbox, AttribID attrib, int index)
         else
           return String(0, NULL);
       }
+      break;
     }
   }
   return String(0, NULL);
