@@ -22,6 +22,7 @@ enum {
   AXIS_LENGTH,
   AXIS_UNIT,
   AXIS_PRETTY,
+  AXIS_USER,
   AXIS_NONE
 };
 
@@ -44,6 +45,7 @@ struct AxisInfo {
   float  unit;
 };
 
+typedef void (*userAxisPtr)(void *userData, int axis, int edge[3]);
 
 class BBoxDeco : public SceneNode 
 {
@@ -58,15 +60,24 @@ public:
   String  getTextAttribute(AABox& bbox, AttribID attrib, int index);
   Material* getMaterial()  { return &material; }
   virtual void getTypeName(char* buffer, int buflen) { strncpy(buffer, "bboxdeco", buflen); };
+  Vec3 marginVecToDataVec(Vec3 marginvec, RenderContext* renderContext, Material* material);
+  Vec3 marginNormalToDataNormal(Vec3 marginvec, RenderContext* renderContext, Material* material);
+  void setAxisCallback(userAxisPtr fn, void * user, int axis);
+  void getAxisCallback(userAxisPtr *fn, void ** user, int axis);
 private:
+  struct BBoxDecoImpl;
   Material material;
   AxisInfo xaxis, yaxis, zaxis;
   float marklen_value;
   bool  marklen_fract;
-#ifndef RGL_NO_OPENGL
   float expand;
-#endif
   bool  draw_front;
+  
+#ifndef RGL_NO_OPENGL
+  bool axisBusy;
+#endif
+  userAxisPtr axisCallback[3];
+  void* axisData[3];
 
   static Material defaultMaterial;
   static AxisInfo defaultAxis;
