@@ -24,6 +24,7 @@ using namespace rgl;
 
 GLboolean GLFont::justify(double twidth, double theight, double adjx, double adjy, 
                           int pos, const RenderContext& rc) {
+#ifndef RGL_NO_OPENGL
   GLdouble pos1[4], pos2[4];
   double basex = 0.0, basey = 0.0, scaling = 1.0;
   GLboolean valid;
@@ -89,6 +90,9 @@ GLboolean GLFont::justify(double twidth, double theight, double adjx, double adj
   
   glGetBooleanv(GL_CURRENT_RASTER_POSITION_VALID, &valid);
   return valid;
+#else
+  return 0;
+#endif
 }
 
 //
@@ -98,13 +102,15 @@ GLboolean GLFont::justify(double twidth, double theight, double adjx, double adj
 
 GLBitmapFont::~GLBitmapFont() {
     delete [] widths;
+#ifndef RGL_NO_OPENGL
     if (nglyph) glDeleteLists(listBase+GL_BITMAP_FONT_FIRST_GLYPH, nglyph);
+#endif
 }
 
 double GLBitmapFont::width(const char* text) {
   double result = 0.0;
   for(int i=0; text[i]; i++) {
-    char c;
+    int c;
     if ((int)(text[i]) >= (int)firstGlyph && (c = (int)(text[i]) - (int)firstGlyph) < (int)nglyph)
       result += widths[(int)c];
   }
@@ -135,20 +141,21 @@ bool GLBitmapFont::valid(const char* text) {
 void GLBitmapFont::draw(const char* text, int length, 
                         double adjx, double adjy, 
                         int pos, const RenderContext& rc) {
-
+#ifndef RGL_NO_OPENGL
   if (justify(width(text), height(), adjx, adjy, pos, rc)) {
     if (rc.gl2psActive == GL2PS_NONE) {
       glListBase(listBase);
       glCallLists(length, GL_UNSIGNED_BYTE, text);
     } else
-      gl2psTextOpt(text, GL2PS_FONT, GL2PS_FONTSIZE*cex, gl2ps_centering, 0.0);
+      gl2psTextOpt(text, GL2PS_FONT, static_cast<GLshort>(GL2PS_FONTSIZE*cex), gl2ps_centering, 0.0);
   }
+#endif
 }
 
 void GLBitmapFont::draw(const wchar_t* text, int length, 
                         double adjx, double adjy, 
                         int pos, const RenderContext& rc) {
-  
+#ifndef RGL_NO_OPENGL  
   if (justify(width(text), height(), adjx, adjy, pos, rc)) {
     GLenum type = sizeof(wchar_t) == 4 ? GL_UNSIGNED_INT :
                   sizeof(wchar_t) == 2 ? GL_UNSIGNED_SHORT : GL_UNSIGNED_BYTE ;
@@ -158,6 +165,7 @@ void GLBitmapFont::draw(const wchar_t* text, int length,
     }
   // gl2ps doesn't support wchar_t?  Should convert?
   }
+#endif
 }
 
 #ifdef HAVE_FREETYPE
@@ -171,7 +179,7 @@ GLFTFont::GLFTFont(const char* in_family, int in_style, double in_cex, const cha
     delete font;
     font = NULL;
   } else {
-    unsigned int size = 16*cex + 0.5;
+    unsigned int size = static_cast<unsigned int>(16*cex + 0.5);
     if (size<1) { size=1; }
     if (!font->FaceSize(size)) {
       errmsg = "Cannot create Freetype font of requested size";
@@ -210,7 +218,7 @@ void GLFTFont::draw(const char* text, int length, double adjx, double adjy,
     if (rc.gl2psActive == GL2PS_NONE)
       font->Render(text);
     else
-      gl2psTextOpt(text, GL2PS_FONT, GL2PS_FONTSIZE*cex, gl2ps_centering, 0.0);
+      gl2psTextOpt(text, GL2PS_FONT, static_cast<GLshort>(GL2PS_FONTSIZE*cex), gl2ps_centering, 0.0);
   }
 }
 
